"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Application = void 0;
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const kill_port_1 = tslib_1.__importDefault(require("kill-port"));
const koa_1 = tslib_1.__importDefault(require("koa"));
const open_1 = tslib_1.__importDefault(require("open"));
const client_1 = require("@/client");
const socket_server_1 = require("@/socket-server");
const log_1 = require("@/utils/log");
const db_1 = require("@/db");
const debug_targets_1 = require("@/controller/debug-targets");
const router_1 = require("@/router");
const config_1 = require("@/config");
const log = new log_1.Logger('application');
class Application {
    static startServer() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            log.info('start server argv: %j', global.appArgv);
            const { host, port, useAdb, useIWDP, useTunnel, open: openChrome, isRemote } = global.appArgv;
            yield Application.init();
            return new Promise((resolve, reject) => {
                const app = new koa_1.default();
                router_1.routeApp(app);
                Application.server = app.listen(port, host, () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    var _a;
                    log.info('start debug server success.');
                    if (!isRemote) {
                        const { startTunnel, startIWDP, startAdbProxy } = yield Promise.resolve().then(() => tslib_1.__importStar(require('./child-process/index')));
                        if (useTunnel)
                            startTunnel();
                        else if (useIWDP)
                            startIWDP();
                        if (useAdb)
                            startAdbProxy();
                        if (openChrome) {
                            const url = `http://${host}:${port}/extensions/home.html`;
                            try {
                                open_1.default(url, { app: { name: open_1.default.apps.chrome } });
                            }
                            catch (e) {
                                log.error('open %s by chrome failed, please open manually, %s', url, (_a = e) === null || _a === void 0 ? void 0 : _a.stack);
                            }
                        }
                    }
                    Application.socketServer = new socket_server_1.SocketServer(Application.server);
                    Application.socketServer.start();
                    resolve(null);
                }));
                Application.server.on('close', () => {
                    log.info('debug server is closed.');
                    reject();
                });
            });
        });
    }
    static stopServer(exitProcess = false) {
        var _a;
        try {
            log.info('stopServer');
            if (Application.server) {
                Application.server.close();
                Application.server = null;
            }
            if (exitProcess)
                process.exit(0);
        }
        catch (e) {
            log.error('stopServer error, %s', (_a = e) === null || _a === void 0 ? void 0 : _a.stack);
        }
    }
    static getDebugTargets() {
        return debug_targets_1.DebugTargetManager.getDebugTargets();
    }
    static init() {
        var _a, _b;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { cachePath } = config_1.config;
            try {
                fs_1.default.rmdirSync(cachePath, { recursive: true });
            }
            catch (e) {
                log.error('rm cache dir error: %s', (_a = e) === null || _a === void 0 ? void 0 : _a.stack);
            }
            yield fs_1.default.promises.mkdir(cachePath, { recursive: true });
            yield db_1.initDbModel();
            client_1.initAppClient();
            const { port, iWDPPort, clearAddrInUse } = global.appArgv;
            if (clearAddrInUse) {
                try {
                    yield kill_port_1.default(port, 'tcp');
                    yield kill_port_1.default(iWDPPort, 'tcp');
                }
                catch (e) {
                    log.error('Address %s %s already in use! %s', port, iWDPPort, (_b = e) === null || _b === void 0 ? void 0 : _b.stack);
                    return process.exit(1);
                }
            }
        });
    }
}
exports.Application = Application;
// 捕获程序退出
process.on('exit', () => Application.stopServer(true));
// 捕获 ctrl c
process.on('SIGINT', () => Application.stopServer(true));
// 捕获 kill
process.on('SIGTERM', () => Application.stopServer(true));
const logErr = (e) => log.error('unhandledRejection %s', e === null || e === void 0 ? void 0 : e.stack);
process.on('unhandledRejection', logErr);
process.on('uncaughtexception', logErr);
