"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onExit = exports.startAdbProxy = exports.startIWDP = exports.startTunnel = exports.tunnelEmitter = exports.TUNNEL_EVENT = void 0;
const tslib_1 = require("tslib");
const child_process_1 = require("child_process");
const events_1 = require("events");
const path_1 = tslib_1.__importDefault(require("path"));
const enum_1 = require("@/@types/enum");
const device_manager_1 = require("@/device-manager");
const log_1 = require("@/utils/log");
const process_1 = require("@/utils/process");
const addon_1 = require("./addon");
const childProcessLog = new log_1.Logger('child-process');
const log = new log_1.Logger('tunnel', enum_1.WinstonColor.Magenta);
let proxyProcess;
exports.TUNNEL_EVENT = 'message';
exports.tunnelEmitter = new events_1.EventEmitter();
const startTunnel = (cb) => {
    const { iWDPPort, iWDPStartPort, iWDPEndPort, adbPath } = global.appArgv;
    addon_1.addEventListener((event, data) => {
        var _a;
        try {
            if (event !== enum_1.TunnelEvent.TunnelLog) {
                log.info('tunnel event: %s', event);
            }
            if (event === enum_1.TunnelEvent.ReceiveData) {
                exports.tunnelEmitter.emit(exports.TUNNEL_EVENT, data);
            }
            else {
                if ([enum_1.TunnelEvent.RemoveDevice, enum_1.TunnelEvent.AddDevice].indexOf(event) !== -1) {
                    device_manager_1.deviceManager.getDeviceList();
                    if (event === enum_1.TunnelEvent.AddDevice) {
                        // 每次设备连接后，运行 adb reverse
                        exports.startAdbProxy();
                    }
                }
                else if (event === enum_1.TunnelEvent.AppConnect) {
                    device_manager_1.deviceManager.onAppConnect();
                }
                else if (event === enum_1.TunnelEvent.AppDisconnect) {
                    device_manager_1.deviceManager.onAppDisconnect();
                }
                else if (event === enum_1.TunnelEvent.TunnelLog && data) {
                    log.info(data);
                }
                if (cb)
                    cb(event, data);
            }
        }
        catch (e) {
            log.error('handle tunnel event error: %s', (_a = e) === null || _a === void 0 ? void 0 : _a.stack);
        }
    });
    let fullAdbPath = adbPath;
    fullAdbPath !== null && fullAdbPath !== void 0 ? fullAdbPath : (fullAdbPath = path_1.default.join(__dirname, '../build/adb'));
    const iWDPParams = ['--no-frontend', `--config=null:${iWDPPort},:${iWDPStartPort}-${iWDPEndPort}`];
    addon_1.tunnelStart(fullAdbPath, iWDPParams, iWDPPort);
};
exports.startTunnel = startTunnel;
const startIWDP = () => {
    const { iWDPPort, iWDPStartPort, iWDPEndPort } = global.appArgv;
    proxyProcess = child_process_1.spawn('ios_webkit_debug_proxy', ['--no-frontend', `--config=null:${iWDPPort},:${iWDPStartPort}-${iWDPEndPort}`], { detached: false });
    proxyProcess.unref();
    childProcessLog.info(`start IWDP on port ${iWDPPort}`);
    proxyProcess.on('error', (e) => {
        childProcessLog.info('IWDP error: %s', e === null || e === void 0 ? void 0 : e.stack);
    });
    proxyProcess.on('close', (code) => {
        childProcessLog.info(`IWDP close with code: ${code}`);
    });
};
exports.startIWDP = startIWDP;
const startAdbProxy = () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const { port } = global.appArgv;
    const adbPath = path_1.default.join(__dirname, '../build/adb');
    try {
        yield process_1.exec(adbPath, ['reverse', '--remove-all']);
        yield process_1.exec(adbPath, ['reverse', `tcp:${port}`, `tcp:${port}`]);
    }
    catch (e) {
        childProcessLog.info('Port reverse failed, For iOS app log.info only just ignore the message.');
        childProcessLog.info('Otherwise please check adb devices command working correctly');
        childProcessLog.info(`type 'adb reverse tcp:${port} tcp:${port}' retry!`);
        childProcessLog.info('start adb reverse error: %s', (_a = e) === null || _a === void 0 ? void 0 : _a.stack);
    }
});
exports.startAdbProxy = startAdbProxy;
const onExit = () => {
    if (!proxyProcess)
        return;
    childProcessLog.info('on log.info server exit, do some clean...');
    proxyProcess === null || proxyProcess === void 0 ? void 0 : proxyProcess.kill('SIGKILL');
    proxyProcess = null;
};
exports.onExit = onExit;
process.on('exit', exports.onExit);
process.on('SIGINT', exports.onExit);
process.on('SIGTERM', exports.onExit);
