"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomainRegister = exports.getDomain = exports.isCdpDomains = exports.CDP_DOMAIN_LIST = void 0;
/**
 * cdp: chrome debug protocol
 */
const events_1 = require("events");
const types_1 = require("tdf-devtools-protocol/dist/types");
const lodash_1 = require("lodash");
exports.CDP_DOMAIN_LIST = lodash_1.uniq(Object.values(types_1.ChromeCommand).map((command) => command.split('.')[0]));
const isCdpDomains = (domain) => exports.CDP_DOMAIN_LIST.indexOf(domain) !== -1;
exports.isCdpDomains = isCdpDomains;
const getDomain = (method) => {
    let domain = method;
    const group = method.match(/^(\w+)(\.\w+)?$/);
    if (group) {
        [, domain] = group;
    }
    return domain;
};
exports.getDomain = getDomain;
class DomainRegister extends events_1.EventEmitter {
    constructor() {
        super(...arguments);
        this.domainListeners = new Map();
        this.registerDomainListener = (domain, listener) => {
            if (!this.domainListeners.has(domain))
                this.domainListeners.set(domain, []);
            this.domainListeners.get(domain).push(listener);
        };
    }
    triggerListerner(msg) {
        if (this.domainListeners.has(msg.method)) {
            this.domainListeners.get(msg.method).forEach((listener) => {
                listener(msg);
            });
        }
    }
}
exports.DomainRegister = DomainRegister;
